<?php

namespace OptimizePress\Integrations\Services\Integrations;

use Exception;
use GuzzleHttp\Client as HttpClient;
use OPDashboard\Services\GeoIp;
use OptimizePress\Integrations\Collections\IntegrationFieldsCollection;
use OptimizePress\Integrations\Collections\IntegrationTagsCollection;
use OptimizePress\Integrations\Collections\IntegrationListsCollection;
use OptimizePress\Integrations\Integration;
use OptimizePress\Integrations\IntegrationTag;
use OptimizePress\Integrations\IntegrationList;
use OptimizePress\Integrations\IntegrationField;
use OptimizePress\Integrations\Integrations;
use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinErrorResult;
use OptimizePress\Integrations\OptinResult;
use OptimizePress\Integrations\OptinSuccessResult;
use OptimizePress\Integrations\Services\Contracts\BaseIntegration;
use OptimizePress\Integrations\Services\Contracts\IntegrationServiceInterface;
use OptimizePress\Integrations\Services\Contracts\IntegrationHasTagsInterface;
use OptimizePress\Integrations\Services\Exceptions\OptinException;
use OptimizePress\Support\Log\Log;

/**
 * Integration with EverWebinar
 */
class MadMimiIntegration extends BaseIntegration implements IntegrationServiceInterface, IntegrationHasTagsInterface
{
    /**
     * Init dependencies and service
     *
     * @param Integration $integration
     */
    public function __construct(Integration $integration)
    {
        $this->init($integration);
        $this->config = [
            'query' => [
                'api_key' => $integration->getConnectionValue('api_key'),
                'username' => $integration->getConnectionValue('email'),
            ],
        ];

        $this->client = new HttpClient($this->config);
    }

    /**
     * Check if integration is ready for access
     *
     * @return bool
     */
    public function isReady()
    {
        if ($this->client) {
            return true;
        }

        return false;
    }

    /**
     * Ping the service API
     *
     * @return bool
     */
    public function ping()
    {
        // https://madmimi.com/api/v3/docs#!/users/users_index
        $url = $this->integration->serviceUrl() . '/users';

        try {
            $response = $this->client->get($url, $this->config);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->users)) {
                return true;
            }
        } catch (Exception $e) {
            Log::error("[MADMINI] Ping failed. " . $e->getMessage());
        }

        return false;
    }

    /**
     * Return web forms
     *
     * @return IntegrationListCollection
     */
    public function getLists()
    {
        // https://madmimi.com/api/v3/docs#!/signupForms/signup_forms_index
        $url = $this->integration->serviceUrl() . '/signupForms';
        $forms = new IntegrationListsCollection;

        try {
            $response = $this->client->get($url, $this->config);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->signupForms)) {
                foreach ($results->signupForms as $form) {
                    $form = $this->listObject([
                        'id' => $form->id,
                        'name' => $form->name,
                        'formFields' => $form->fields,
                    ]);
                    $forms->push($form);
                }

                return $forms;
            }

            Log::error('[MADMINI] Failed to get forms.');
        } catch (Exception $e) {
            Log::error('[MADMINI] Error when fetching forms from service. ' . $e->getMessage());
        }

        return $forms;

    }

    /**
     * Return form
     *
     * @param  string  $id
     * @return IntegrationList
     */
    public function getList($id = null)
    {
        $forms = $this->getLists();

        if ($forms) {
            foreach ($forms as $form) {
                if ($form->id == $id) {
                    return $form;
                }
            }
        }

        return null;
    }

    /**
     * Return all lists
     *
     * @return IntegrationListsCollection
     */
    public function getTags()
    {
        // https://madmimi.com/api/v3/docs#!/subscriberLists/subscriber_lists_index
        $url = $this->integration->serviceUrl() . '/subscriberLists';
        $lists = new IntegrationListsCollection;

        try {
            $response = $this->client->get($url, $this->config);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->subscriberLists)) {
                foreach ($results->subscriberLists as $list) {
                    $list = $this->listObject([
                        'id'   => $list->id,
                        'name' => $list->name,
                    ]);
                    $lists->push($list);
                }

                return $lists;
            }

            Log::error('[MADMINI] Failed to get lists.');
        } catch (Exception $e) {
            Log::error('[MADMINI] Error when fetching subscriber lists from service. ' . $e->getMessage());
        }

        return $lists;
    }

    /**
     * Return a list
     *
     * @param  string  $id
     * @return IntegrationTag
     */
    public function getTag($id = null)
    {
        $lists = $this->getTags();

        if ($lists) {
            foreach ($lists as $list) {
                if ($list->id == $id) {
                    return $list;
                }
            }
        }

        return null;
    }

    /**
     * Get webform fields.
     *
     * @param  string  $webFormId
     * @return Integratilistsdollection
     */
    public function getFields($webFormId = null)
    {
        if (! $webFormId)
            return;

        $form = $this->getList($webFormId);
        $fields = new IntegrationFieldsCollection;
        $fieldsOrder = [ 'first_name' => 10, 'last_name' => 20 ];

        // Add Email field
        $fields->push(new IntegrationField(['id' => 'email', 'name' => 'email', 'label' => 'E-Mail', 'optin_id' => 'email', 'type' => 'email', 'enabled'  => true, 'required' => true, 'order' => 30]));

        // Add all other fields
        foreach($form->formFields as $field) {
            $notSupportedFields = [ 'radio_buttons', 'checkboxes', 'tos_link' ];
            $type = null;

            if (in_array($field->type, $notSupportedFields)) {
                continue;
            } elseif ($field->type === "dropdown") {
                $type = 'select';
            } elseif ($field->type === "age_check" || $field->type === 'tracking_option') {
                $type = 'checkbox';
            } elseif ($field->type === "date") {
                $type = 'date';
            } else {
                $type = 'text';
            }

            $integrationField = new IntegrationField([
                'id' => $field->name,
                'name' => $field->name,
                'label' => $field->displayName ? $field->displayName : ucwords(str_replace('_', ' ', $field->name)),
                'optin_id' => lcfirst(str_replace('_', '', ucwords($field->name, '_'))),
                'type' => $type,
                'enabled' => true,
                'required' => $field->required ? $field->required : false,
                'order' => isset($fieldsOrder[$field->name]) ? $fieldsOrder[$field->name] : 40,
            ]);

            // Add select options
            if ($type === "select") {
                foreach ($field->options as $option) {
                    $integrationField->addOptionValue($option, $option);
                }
            }

            $fields->push($integrationField);
        }

        return $fields;
    }

    /**
     * Updates subscriber's list memberships
     *
     * @param string $subscriberId
     * @param string $listId
     * @return bool
     * @throws OptinException
     */
    public function updateSubscriberList($subscriberId, $listId)
    {
        if (!isset($subscriberId) && !isset($listId))
            return;

        // https://madmimi.com/api/v3/docs#!/subscribers/subscriber_create
        $url = $this->integration->serviceUrl() . '/subscribers/' . $subscriberId . '/memberships/';
        $data = [
            'json' => [
                'add' => [
                    $listId,
                ]
            ]
        ];

        try {
            $response = $this->client->put($url, $data);

            if ($response->getStatusCode() === 204) {
                return true;
            }

            Log::error("[MADMINI] Failed to add subscriber [$subscriberId] to list [$listId].");
        } catch (Exception $e) {
            Log::error("[MADMINI] Error while adding subscriber [$subscriberId] to list [$listId]." . $e->getMessage());
            $this->throwOptinException($e);
        }

        return false;
    }

    /**
     * Create new subscriber
     *
     * @param array $data
     * @return mixed
     * @throws OptinException
     */
    public function createSubscriber($data)
    {
        // https://madmimi.com/api/v3/docs#!/subscribers/subscriber_create
        $url = $this->integration->serviceUrl() . '/subscribers';

        try {
            $response = $this->client->post($url, $data);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->subscriber)) {
                return $results->subscriber;
            }

            Log::error("[MADMINI] Failed to create subscriber.");
        } catch (Exception $e) {
            Log::error("[MADMINI] Error while creating subscriber." . $e->getMessage());
            $this->throwOptinException($e);
        }

        return false;
    }

    /**
     * Optin to email service
     *
     * @param  OptinData   $optinData
     * @param  LeadOptions $leadOptions
     * @return OptinResult
     */
    public function optin(OptinData $optinData, LeadOptions $leadOptions = null)
    {
        $result = $this->optinErrorResult();
        $listId = $optinData->listId();
        $fields = $this->getFields($listId);
        $data = $this->config;

        // Prepare fields (store extra fields into auxData)
        foreach ($fields as $field) {
            if ($value = $optinData->data($field->id)) {
                if ($field->type !== 'text' && $field->type !== 'email')
                    $data['json']['auxData'][$field->optin_id] = $value;
                else
                    $data['json'][$field->optin_id] = $value;
            }
        };

        // List is stored as tag,
        // because webform is stored as list
        $lists = $optinData->tags();
        if (isset($lists) && !empty($lists))
            $data['json']['listId'] = $lists[0];

        // GDPR FromEu
        if (class_exists('\OPDashboard\Services\GeoIp')) {
            $optinData->isFromEu = GeoIp::isFromEu();
        } else {
            $optinData->isFromEu = true;
        }

        // GDPR
        $gdpr = $this->processGdprTags($optinData, true);
        if (! empty($gdpr)) {
            foreach ($gdpr as $key => $value) {
                $data['json']['auxData'][$value['id']] = 'yes';
            }
        }

        // GDPR note
        $gdprNote = $this->getGdprNote($optinData);

        if (! empty($gdprNote) && !empty($optinData->get('optin-gdpr-field-note'))) {
            $data['json']['auxData'][$optinData->get('optin-gdpr-field-note')] = $gdprNote;
        }

        $lead = $this->createSubscriber($data);
        if ($lead) {
            $added = $this->updateSubscriberList($lead->id, $data['json']['listId']);

            if ($added)
                $result = $this->optinSuccessResult();
        }

        return $result;
    }

    /**
     * Throw an optin exception
     *
     * @param  string|Exception   $exception
     * @param  array              $data
     * @throws OptinException
     */
    public function throwOptinException($exception = null, $data = [])
    {
        // Parse the exception
        if (is_string($exception)) {
            $message = $exception;
            $code    = 500;
        } elseif ($exception instanceof Exception) {
            $message = $exception->getMessage();
            $code = $exception->getCode();

            // Get error message from truncated body (invalid json)
            // This way we display more appropriate message to user
            $regex = '/(?:\"|\')message(?:\"|\')(?=:)(?:\:\s*)(?:\"|\')?(true|false|[0-9a-zA-Z\s@_\+\-\,\.\$]*)/';
            preg_match_all($regex, $message, $matches, PREG_SET_ORDER, 0);

            $message = "Error code: " . $code . ". ";
            if (isset($matches)) {
                foreach ($matches as $match)
                    $message .= (isset($match[1]) ? $match[1] : "") . ". ";
            }

        } else {
            $message = 'Failed to optin.';
            $code    = 500;
        }

        // And throw the exception
        throw new OptinException($message, array_merge([
            'code'        => $code,
            'optinData'   => $this->optinData,
            'leadOption'  => $this->leadOptions,
            'integration' => $this,
        ], $data));
    }

}
