<?php

namespace OptimizePress\Integrations\Services\Integrations;

use Exception;
use \DateTimeZone;
use \DateTime;
use OPDashboard\Services\GeoIp;
use GuzzleHttp\Client as HttpClient;
use OptimizePress\Integrations\Collections\IntegrationFieldsCollection;
use OptimizePress\Integrations\Collections\IntegrationListsCollection;
use OptimizePress\Integrations\Integration;
use OptimizePress\Integrations\IntegrationList;
use OptimizePress\Integrations\IntegrationField;
use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinErrorResult;
use OptimizePress\Integrations\OptinResult;
use OptimizePress\Integrations\OptinSuccessResult;
use OptimizePress\Integrations\Services\Contracts\BaseIntegration;
use OptimizePress\Integrations\Services\Contracts\IntegrationServiceInterface;
use OptimizePress\Integrations\Services\Exceptions\AlreadySubscribedException;
use OptimizePress\Integrations\Services\Exceptions\OptinException;
use OptimizePress\Support\Log\Log;

/**
 * Integration with WebinarJam
 */
class WebinarJamIntegration extends BaseIntegration implements IntegrationServiceInterface
{
    /**
     * Number of seconds in one hour
     *
     * @var int SECCONDS_IN_HOUR
     */
    const SECONDS_IN_HOUR = 3600;

    /**
     * Init dependencies and service
     *
     * @param Integration $integration
     */
    public function __construct(Integration $integration)
    {
        // Initialize and setup
        $this->init($integration);

        // Setup base config
        $this->config = [
            'api_key' => $integration->getConnectionValue('api_key'),
        ];

        // HTTP clients and helpers
        $this->client = new HttpClient($this->config);
    }

    /**
     * Check if integration is ready for access
     *
     * @return bool
     */
    public function isReady()
    {
        if ($this->client) {
            return true;
        }

        return false;
    }

    /**
     * Ping the service API
     *
     * @return bool
     */
    public function ping()
    {
        // https://documentation.webinarjam.com/retrieve-a-full-list-of-all-webinars-published-in-your-account/
        $url = $this->integration->serviceUrl() . '/webinars';
        $data = [
            'query' => [
                'api_key' => $this->config['api_key'],
            ]
        ];

        try {
            $response = $this->client->post($url, $data);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->status)) {
                return true;
            }
        } catch (Exception $e) {
            Log::error("[WEBINARJAM] Ping failed. " . $e->getMessage());
        }

        return false;
    }

    /**
     * Return all lists
     *
     * @return IntegrationListsCollection
     */
    public function getLists()
    {
        // https://documentation.webinarjam.com/retrieve-a-full-list-of-all-webinars-published-in-your-account/
        $url = $this->integration->serviceUrl() . '/webinars';
        $data = [
            'query' => [
                'api_key' => $this->config['api_key'],
            ]
        ];
        $lists = new IntegrationListsCollection;

        try {
            $response = $this->client->post($url, $data);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->webinars)) {
                foreach ($results->webinars as $webinar) {
                    // Only add single presentation webinar type
                    $type = strtolower($webinar->type);
                    if (in_array($type, ['single presentation', 'series of presentations'])) {
                        $list = $this->listObject([
                            'id'   => $webinar->webinar_id,
                            'name' => $webinar->name,
                            'type' => $webinar->type,
                        ]);
                        $lists->push($list);
                    }
                }

                return $lists;
            }

            Log::error('[WEBINARJAM] Failed to get lists.');
        } catch (Exception $e) {
            Log::error('[WEBINARJAM] Error when fetching lists from service. ' . $e->getMessage());
        }

        return $lists;
    }

    /**
     * Get a webinar
     *
     * @param  string  $listId
     * @return array
     */
    public function getList($listId)
    {
        if (! $listId)
            return [];

        $data = [
            'query' => [
                'api_key' => $this->config['api_key'],
                'webinar_id' => $listId,
            ]
        ];

        // https://documentation.webinarjam.com/get-details-about-one-particular-webinar-from-your-account/
        $url = $this->integration->serviceUrl() . '/webinar';

        try {
            $response = $this->client->post($url, $data);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->webinar) and isset($results->webinar->schedules)) {
                return $results->webinar;
            }

            Log::debug("[WEBINARJAM] Webinar schedules fetched on webinar [$listId].");
        } catch (Exception $e) {
            Log::error("[WEBINARJAM] Error when fetching webinar schedules [$listId] from service. " . $e->getMessage());
        }

        return [];
    }

    /**
     * Get webinar and convert dates in new timezone.
     * It's same as getList method,
     * only difference is timezone support.
     *
     * @param string $listId
     * @param string $timezone
     * @return array
     */
    public function getWebinar($listId = null, $timezone = null)
    {
        $webinar = $this->getList($listId);

        if ($webinar && isset($webinar->schedules) && $timezone) {
            foreach($webinar->schedules as $schedule) {
                // Init dates in orginal timezone
                $orginalTimeZone = new DateTimeZone($webinar->timezone);
                $date = new DateTime($schedule->date, $orginalTimeZone);

                // Convert from orginal timezone to new timezone
                $newTimeZone = new DateTimeZone($timezone);
                $date->setTimeZone($newTimeZone);
                $schedule->date = $date->format('Y-m-d H:i:s');
                $schedule->formatted = $date->format('l, M j Y, g:i A');
            }

            $webinar->timezone = $timezone;
        }

        return $webinar;
    }

    /**
     * Get form fields.
     * Fields are hardcoded based on api parameters
     * when registring user to webinar.
     *
     * @param  string  $listId
     * @return IntegrationFieldsCollection
     */
    public function getFields($listId = null)
    {
        $fields = new IntegrationFieldsCollection;

        if (! $listId)
            return $fields;

        // First Name
        $fields->push(new IntegrationField([ 'id' => 'first_name', 'name' => 'first_name', 'label' => 'First Name', 'optin_id' => 'first_name', 'enabled' => true, 'required' => true, 'order' => 0, 'type' => 'text', ]));

        // Last Name
        $fields->push(new IntegrationField([ 'id' => 'last_name', 'name' => 'last_name', 'label' => 'Last Name', 'optin_id' => 'last_name', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'text', ]));

        // Email
        $fields->push(new IntegrationField([ 'id' => 'email', 'name' => 'email', 'label' => 'E-Mail', 'optin_id' => 'email', 'enabled' => true, 'required' => true, 'order' => 0, 'type' => 'email', ]));

        // Phone
        $fields->push(new IntegrationField([ 'id' => 'phone', 'name' => 'phone', 'label' => 'Phone', 'optin_id' => 'phone', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'text', ]));

        // Phone Country - Get codes from config file and fill field
        $field = new IntegrationField([ 'id' => 'phone_country_code', 'name' => 'phone_country_code', 'label' => 'Phone Country Code', 'optin_id' => 'phone_country_code', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'select', ]);
        $countryPhoneCodes = include(__DIR__ . '/../../../config/country_phone_codes.php');
        foreach ($countryPhoneCodes as $code) {
            $field->addOptionValue($code['code'], $code['name']);
        }
        $fields->push($field);

        // Timezone - Fill field from php timezone list
        $field = new IntegrationField([ 'id' => 'timezone', 'name' => 'timezone', 'label' => 'Timezone', 'optin_id' => 'timezone', 'enabled' => false, 'required' => false, 'order' => 0, 'type' => 'select', ]);
        $timezones = DateTimeZone::listIdentifiers(DateTimeZone::ALL);
        foreach ($timezones as $timezone) {
            $field->addOptionValue($timezone, $timezone);
        }
        $fields->push($field);

        // Schedules (updated with option values below)
        $fields->push(new IntegrationField([ 'id' => 'schedule', 'name' => 'schedule', 'label' => 'Webinar Schedule', 'optin_id' => 'schedule', 'enabled' => true, 'required' => true, 'order' => 0, 'type' => 'select', ]));

        // Date
        $fields->push(new IntegrationField([ 'id' => 'date', 'name' => 'date', 'label' => 'Date', 'optin_id' => 'date', 'enabled' => false, 'required' => true, 'order' => 0, 'type' => 'hidden', ]));

        return $fields;
    }

    /**
     * Optin to email service
     *
     * @param  OptinData   $optinData
     * @param  LeadOptions $leadOptions
     * @return OptinResult
     * @throws OptinException
     */
    public function optin(OptinData $optinData, LeadOptions $leadOptions = null)
    {
        $result = $this->optinErrorResult();
        $listId = $optinData->listId();
        $fields = $this->getFields($listId);

        $data = [
            'query' => [
                'api_key' => $this->config['api_key'],
                'webinar_id' => $listId,
            ]
        ];

        // Prepare fields
        foreach ($fields as $field) {
            if ($value = $optinData->data($field->id)) {
                $data['query'][$field->optin_id] = $value;
            }
        };

        // Convert IANA timezone name to GMT
        if (isset($data['query']['timezone'])) {
            $zone = new DateTimeZone($data['query']['timezone']);
            $time = new DateTime($data['query']['date']);
            $offsetInSeconds = $zone->getOffset($time);
            $offsetInHour = $offsetInSeconds / self::SECONDS_IN_HOUR;
            $prefix = $offsetInHour > 0 ? '+' : '-';

            $data['query']['timezone'] = "GMT" . $prefix . abs($offsetInHour);
        }

        // https://documentation.webinarjam.com/register-a-person-to-a-specific-webinar/
        $url = $this->integration->serviceUrl() . '/register';

        try {
            $response = $this->client->post($url, $data);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->user)) {
                $result = $this->optinSuccessResult("Optin Success!", [ 'responseData' => $results ]);
            } else {
                Log::error("[WEBINARJAM] Failed to subscribe user to webinar [$listId].");
                $this->throwOptinException();
            }
        } catch (Exception $e) {
            Log::error("[WEBINARJAM] Error when subscribing to webinar [$listId]. " . $e->getMessage());
            $this->throwOptinException($e);
        }

        return $result;
    }
}
