<?php

namespace OptimizePress\Integrations\Services\Membership;

use OPCart\Model\Settings\Email\UserCredentials;
use OPCart\Repositories\EmailRepository;
use OptimizePress\Integrations\Collections\IntegrationFieldsCollection;
use OptimizePress\Integrations\Integration;
use OptimizePress\Integrations\IntegrationField;
use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinErrorResult;
use OptimizePress\Integrations\OptinResult;
use OptimizePress\Integrations\OptinSuccessResult;
use OptimizePress\Integrations\Services\Contracts\BaseProvider;
use OptimizePress\Integrations\Services\Exceptions\OptinException;
use OptimizePress\Support\Log\Log;

abstract class BaseIntegration
{
    /**
     * Service integration
     *
     * @var MembershipIntegration
     */
    protected $integration;

    /**
     * Service client
     *
     * @var mixed
     */
    protected $client;

    /**
     * Basic configuration
     *
     * @var array
     */
    protected $config;

    /**
     * @var BaseProvider
     */
    protected $provider;

    /**
     * @var OptinData
     */
    protected $optinData;

    /**
     * @var LeadOptions
     */
    protected $leadOptions;

    /**
     * Init new integration provider
     *
     * @param MembershipIntegration $integration
     */
    protected function init(MembershipIntegration $integration)
    {
        $this->integration = $integration;
        $this->config      = &$integration->config;
        $this->provider    = &$integration->provider;
    }

    /**
     * Send email with user credentials
     *
     * @param  $user
     * @param null $password
     * @return bool
     */
    public function sendUserNotification($user, $password = null)
    {
        // Setup email
        error_log(print_r($user, true));
        $to      = $user->user_email;

        // setup email data
        $subject = 'Important - Access Credentials';
        if ($user->is_new) {
            $body = "Hi {firstname},

Thank you for your subscription.

Your login credentials are below.

======================

Your E-Mail: {email}
Your Password: {password}

======================

We recommend that you save and keep these details safe.

If you require assistance, please don't hesitate to contact our friendly support team who will be happy to help.

You can reach us at: {siteemail}

Thanks again.

Exciting times ahead - {sitename} Team";
        } else {
            $body = "Hi {firstname},

Thank you for your subscription.

You had previous access to {sitename}, so please use your existing credentials.

======================

Your E-Mail: {email}

======================

If you require assistance, please don't hesitate to contact our friendly support team who will be happy to help.

You can reach us at: {siteemail}

Thanks again.

Exciting times ahead - {sitename} Team";
        }

        // Insert data
        $body = str_replace('{email}',     $user->user_email, $body);
        $body = str_replace('{firstname}', $user->first_name, $body);
        $body = str_replace('{password}',  $password,         $body);
        $body = str_replace('{siteemail}', get_bloginfo('admin_email'), $body);
        $body = str_replace('{sitename}',  get_bloginfo('name'), $body);

        return wp_mail($to, $subject, $body);
    }

    /**
     * Get form fields
     *
     * @return IntegrationFieldsCollection
     */
    public function getFields(): IntegrationFieldsCollection
    {
        $fields = new IntegrationFieldsCollection;

        // Add default fields
        $field = new IntegrationField(['id' => 'first_name', 'name' => 'first_name', 'label' => 'First Name', 'optin_id' => 'first_name', 'type' => 'text',  'enabled'  => false, 'required' => false, 'order' => 10]);
        $fields->push($field);

        $field = new IntegrationField(['id' => 'last_name',  'name' => 'last_name',  'label' => 'Last Name',  'optin_id' => 'last_name',  'type' => 'text',  'enabled'  => false, 'required' => false, 'order' => 20]);
        $fields->push($field);
        $field = new IntegrationField(['id' => 'email',      'name' => 'email',      'label' => 'E-Mail',     'optin_id' => 'email',      'type' => 'email', 'enabled'  => true,  'required' => true,  'order' => 30]);
        $fields->push($field);

        // Check if all the fields are valid for display
        return $fields;
    }

    /**
     * User gets access to specific membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return OptinResult
     * @throws OptinException
     */
    public function optin(OptinData $optinData, LeadOptions $leadOptions = null) :OptinResult
    {
        return $this->grantLevelAccess($optinData, $leadOptions);
    }

    /**
     * Prepare optin success result object
     *
     * @param  string  $message
     * @param  array   $data
     * @return OptinSuccessResult
     */
    public function optinSuccessResult($message = null, $data = []): OptinSuccessResult
    {
        return new OptinSuccessResult(array_merge([
            'message'     => $message,
            'optinData'   => $this->optinData,
            'leadOption'  => $this->leadOptions,
            'integration' => $this,
        ], $data));
    }

    /**
     * Prepare optin success result object
     *
     * @param  string  $message
     * @param  array   $data
     * @return OptinErrorResult
     */
    public function optinErrorResult($message = null, $data = []): OptinErrorResult
    {
        return new OptinErrorResult(array_merge([
            'message'     => $message,
            'optinData'   => $this->optinData,
            'leadOption'  => $this->leadOptions,
            'integration' => $this,
        ], $data));
    }
}
