<?php

namespace OptimizePress\Integrations\Services\Membership\Services;

use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinResult;
use OptimizePress\Integrations\Services\Membership\BaseIntegration;
use OptimizePress\Integrations\Services\Membership\MembershipIntegration;
use OptimizePress\Integrations\Services\Membership\MembershipIntegrationInterface;
use OptimizePress\Integrations\Services\Membership\MembershipLevel;
use OptimizePress\Integrations\Services\Membership\Vendor\WLMApi;
use OptimizePress\Support\Collection;
use OptimizePress\Support\Log\Log;

class WishListMemberIntegration extends BaseIntegration implements MembershipIntegrationInterface
{
    /**
     * @var WLMApi
     */
    protected $client;

    /**
     * @var string
     */
    protected $apiKey;

    /**
     * Init new membership integration
     *
     * @param  MembershipIntegration  $integration
     */
    public function __construct(MembershipIntegration $integration)
    {
        // Initialize and setup
        $this->init($integration);

        if ($apiKey = $this->apiKey()) {
            $this->client = new WLMApi(home_url(), $apiKey);
            $this->client->return_format = 'json';
        }
    }

    /**
     * Check if integration is active
     *
     * @return bool
     */
    public function isActive(): bool
    {
        return ($this->apiKey() && class_exists(\WishListMember3::class));
    }

    /**
     * Get all levels for the membership integration
     *
     * @return Collection
     */
    public function getLevels() :Collection
    {
        Log::debug('[OPCART] [WLM] Fetching access levels.');
        $levels = new Collection;

        // Get the levels
        $response = $this->client->get('/levels');
        Log::debug('[OPCART] [WLM] Access levels response: ' . $response);
        if ($response && $result = @json_decode($response)) {
            foreach ($result->levels->level as $accessLevel) {
                $levels->push(new MembershipLevel([
                    'id'    => $accessLevel->id,
                    'name'  => $accessLevel->name,
                    'title' => $accessLevel->name,
                ]));
            }
        } else {
            Log::error('[OPCART] [WLM] Failed fetching access levels.');
        }

        return $levels;
    }

    /**
     * User gets access to specific membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return OptinResult
     */
    public function grantLevelAccess(OptinData $optinData, LeadOptions $leadOptions = null) :OptinResult
    {
        // Setup params
        $level  = $optinData->listId();
        $result = $this->optinErrorResult();

        if ($level) {
            Log::debug('[OPCART] [WLM] Granting access to level: ' . $level);

            // Find the level data
            $levelDataResponse = $this->client->get('/levels/' . $level);
            $levelData         = @json_decode($levelDataResponse);

            if ($levelData && $levelData->success) {
                $password = wp_generate_password(12, false);

                // Try to find the user
                $user = get_user_by('email', $optinData->email());

                // If user is not found we create a new one
                if (! $user) {
                    $userId         = wp_create_user($optinData->email(), $password, $optinData->email());
                    $user           = get_user_by('id', $userId);
                    $user->is_new   = true;
                    $user->password = $password;

                    // Update first/last name if needed
                    if ($user) {
                        if ($optinData->firstName()) wp_update_user(['ID' => $user->ID, 'first_name' => $optinData->firstName()]);
                        if ($optinData->lastName())  wp_update_user(['ID' => $user->ID, 'last_name'  => $optinData->lastName()]);
                    }
                } else {
                    // user existed, let us add data about his previous RUA level
                    $userData = $this->client->get('/members/' . $user->ID);
                    $userData = @json_decode($userData);
                    Log::debug('[OPCART] Existing user data ' . print_r($userData, true));
                    $user->previousLevel = $userData->member[0]->Levels;
                }

                // Add user to membership level
                if ($user && isset($user->ID)) {
                    $accessResponse = $this->client->post('/levels/' . $level . '/members', [
                        'Users' => [$user->ID],
                    ]);
                    $accessResponseData = @json_decode($accessResponse);

                    if ($accessResponseData && $accessResponseData->success) {
                        Log::debug('[OPCART] [WLM] Access to level [' . $level . '] granted to user [' . $user->ID . ']');

                        if (! $optinData->fromCart) {
                            $this->sendUserNotification($user, $password);
                        }
                        $result = $this->optinSuccessResult('Access granted.', ['user' => $user]);
                    } else {
                        Log::error('[OPCART] [WLM] Error when granting access to level [' . $level . '] for user [' . $user->ID . ']');
                    }
                }
            }
        }

        return $result;
    }

    /**
     * Revoke access for user to membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return bool
     */
    public function revokeLevelAccess(OptinData $optinData, LeadOptions $leadOptions = null) :bool
    {
        // Setup params
        $level = $optinData->listId();

        if ($level) {
            Log::debug('[OPCART] [WLM] Revoking access to level: ' . $level);

            // Find the level data
            $levelDataResponse = $this->client->get('/levels/' . $level);
            $levelData         = @json_decode($levelDataResponse);

            if ($levelData && $levelData->success) {
                // Try to find the user
                $user = get_user_by('email', $optinData->email());

                // Add user to membership level
                if ($user && isset($user->ID)) {
                    $accessResponse = $this->client->delete('/levels/' . $level . '/members/' . $user->ID);
                    $accessResponseData = @json_decode($accessResponse);

                    if ($accessResponseData && $accessResponseData->success) {
                        Log::debug('[OPCART] [WLM] Access to level [' . $level . '] revoked to user [' . $user->ID . ']');

                        return true;
                    }

                    Log::error('[OPCART] [WLM] Error when revoking access to level [' . $level . '] for user [' . $user->ID . ']');
                }
            }
        }

        return false;
    }

    /**
     * Fetch WLM API key from options
     *
     * @return string
     */
    public function apiKey()
    {
        global $wpdb;

        if (! $this->apiKey && class_exists('\WishListMember3')) {
            // Get the API key
            $apiKeyRow = $wpdb->get_row($wpdb->prepare("SELECT `option_value` FROM `{$wpdb->prefix}wlm_options` WHERE `option_name`='%s'", 'WLMAPIKey'));

            if ($apiKeyRow && isset($apiKeyRow->option_value)) {
                $this->apiKey = $apiKeyRow->option_value;
            }

            Log::debug('[OPCART] [WLM] Fetching API key: ' . $this->apiKey);
        }


        return $this->apiKey;
    }
}
